@@ -532,6 +532,16 @@ class FocusTrackingPresenter(presentation_manager.PresentationManager):
                     self._cleanupGarbage()
             return
 
+        # Clean up any flat review context so that Orca does not get
+        # confused (see bgo#609633)
+        #
+        if event.type.startswith("window:deactivate") \
+           and orca_state.activeScript \
+           and orca_state.activeScript.flatReviewContext \
+           and orca_state.activeScript.app == event.host_application:
+            orca_state.activeScript.drawOutline(-1, 0, 0, 0)
+            orca_state.activeScript.flatReviewContext = None
+
         try:
             # We don't want to touch a defunct object.  It's useless and it
             # can cause hangs.
@@ -542,10 +552,6 @@ class FocusTrackingPresenter(presentation_manager.PresentationManager):
                     debug.println(debug.LEVEL_FINEST,
                                   "IGNORING DEFUNCT OBJECT")
                     if event.type.startswith("window:deactivate"):
-                        if orca_state.activeScript \
-                           and orca_state.activeScript.flatReviewContext:
-                            orca_state.activeScript.drawOutline(-1, 0, 0, 0)
-                            orca_state.activeScript.flatReviewContext = None
                         orca.setLocusOfFocus(event, None)
                         orca_state.activeWindow = None
                     return
@@ -811,11 +817,12 @@ class FocusTrackingPresenter(presentation_manager.PresentationManager):
                 debug.println(debug.LEVEL_ALL,
                               "---------> QUEUEING EVENT %s" % e.type)
 
-            # Some toolkits (e.g., Java - see bug #531869) need to
+            # Some toolkits (e.g., Java - see bug #531869) may need to
             # have their events processed immediately.
             #
             try:
-                if event.host_application.toolkitName \
+                if len(settings.synchronousToolkits) > 0 and \
+                    event.host_application.toolkitName \
                     in settings.synchronousToolkits:
                     asyncMode = False
             except:
