@@ -1,6 +1,7 @@
 # Orca
 #
 # Copyright 2006-2009 Sun Microsystems Inc.
+# Copyright 2010 Joanmarie Diggs
 #
 # This library is free software; you can redistribute it and/or
 # modify it under the terms of the GNU Library General Public
@@ -20,7 +21,8 @@
 __id__        = "$Id$"
 __version__   = "$Revision$"
 __date__      = "$Date$"
-__copyright__ = "Copyright (c) 2005-2009 Sun Microsystems Inc."
+__copyright__ = "Copyright (c) 2005-2009 Sun Microsystems Inc., "  \
+                "Copyright (c) 2010 Joanmarie Diggs"
 __license__   = "LGPL"
 
 import pyatspi
@@ -29,9 +31,7 @@ import orca.default as default
 import orca.input_event as input_event
 import orca.orca as orca
 import orca.orca_state as orca_state
-import orca.keybindings as keybindings
 
-from braillegenerator import BrailleGenerator
 from speech_generator import SpeechGenerator
 from formatting import Formatting
 
@@ -51,13 +51,17 @@ class Script(default.Script):
         """
         default.Script.__init__(self, app)
 
-    def getBrailleGenerator(self):
-        """Returns the braille generator for this script.
-        """
-        return BrailleGenerator(self)
+        # Some objects which issue descendant changed events lack
+        # STATE_MANAGES_DESCENDANTS. As a result, onSelectionChanged
+        # doesn't ignore these objects. That in turn causes Orca to
+        # double-speak some items and/or set the locusOfFocus to a
+        # parent it shouldn't. See bgo#616582. [[[TODO - JD: remove
+        # this hack if and when we get a fix for that bug]]]
+        # 
+        self.lastDescendantChangedSource = None
 
     def getSpeechGenerator(self):
-        """Returns the braille generator for this script.
+        """Returns the speech generator for this script.
         """
         return SpeechGenerator(self)
 
@@ -65,30 +69,71 @@ class Script(default.Script):
         """Returns the formatting strings for this script."""
         return Formatting(self)
 
-    def consumesKeyboardEvent(self, keyboardEvent):
-        """Called when a key is pressed on the keyboard.
+    def checkKeyboardEventData(self, keyboardEvent):
+        """Checks the data on the keyboard event.
+
+        Some toolkits don't fill all the key event fields, and/or fills
+        them out with unexpected data. This method tries to fill in the
+        missing fields and validate/standardize the data we've been given.
+        While any script can override this method, it is expected that
+        this will only be done at the toolkit script level.
 
         Arguments:
         - keyboardEvent: an instance of input_event.KeyboardEvent
-
-        Returns True if the event is of interest.
         """
 
-        # The Java platform chooses to give us keycodes different from
-        # the native platform keycodes.  So, we hack here by converting
-        # the keysym we get from Java into a keycode.
+        default.Script.checkKeyboardEventData(self, keyboardEvent)
 
-        keysym = keyboardEvent.event_string
+        if not keyboardEvent.keyval_name:
+            return
 
-        # We need to make sure we have a keysym-like thing.  The space
-        # character is not a keysym, so we convert it into the string,
-        # 'space', which is.
+        import gtk.gdk as gdk
+
+        # Standardize the hw_code from a Java-unique one to a gdk one.
         #
-        if keysym == " ":
-            keysym = "space"
+        keymap = gdk.keymap_get_default()
+        keyval = gdk.keyval_from_name(keyboardEvent.keyval_name)
+        entries = keymap.get_entries_for_keyval(keyval)
+        for entry in entries:
+            if entry[1] == 0:  # group = 0
+                keyboardEvent.hw_code = entry[0]
+                break
 
-        keyboardEvent.hw_code = keybindings.getKeycode(keysym)
-        return default.Script.consumesKeyboardEvent(self, keyboardEvent)
+        # Put the event_string back to what it was prior to the Java
+        # Atk Wrapper hack which gives us the keyname and not the
+        # expected and needed printable character for punctuation
+        # marks.
+        #
+        if keyboardEvent.event_string == keyboardEvent.keyval_name \
+           and len(keyboardEvent.event_string) > 1:
+            keyval = gdk.keyval_from_name(keyboardEvent.keyval_name)
+            if 0 < keyval < 256:
+                keyboardEvent.event_string = unichr(keyval).encode("UTF-8")
+
+    def getValidObj(self, rootObj, obj, onlyShowing=True):
+        """Attempts to convert an older copy of an accessible into the
+        current, active version. We need to do this in order to ascend
+        the hierarchy.
+
+        Arguments:
+        - rootObj: the top-most ancestor of interest
+        - obj: the old object we're attempting to replace
+        - onlyShowing: whether or not we should limit matches to those
+          which have STATE_SHOWING
+
+         Returns an accessible replacement for obj if one can be found;
+         otherwise, None.
+         """
+
+        if not (obj and rootObj):
+            return None
+
+        items = self.findByRole(rootObj, obj.getRole(), onlyShowing)
+        for item in items:
+            if item.name == obj.name and self.isSameObject(item, obj):
+                return item
+
+        return None
 
     def getNodeLevel(self, obj):
         """Determines the node level of this object if it is in a tree
@@ -99,30 +144,52 @@ class Script(default.Script):
         -obj: the Accessible object
         """
 
-        if not obj:
-            return -1
-
-        treeLikeThing = self.getAncestor(obj,
-                                         [pyatspi.ROLE_TREE,
-                                          pyatspi.ROLE_TABLE,
-                                          pyatspi.ROLE_TREE_TABLE],
-                                         None)
-        if not treeLikeThing:
-            return -1
+        newObj = self.getValidObj(self.lastDescendantChangedSource, obj)
+        if not newObj:
+            return default.Script.getNodeLevel(self, obj)
 
         count = 0
-        while True:
-            state = obj.getState()
+        while newObj:
+            state = newObj.getState()
             if state.contains(pyatspi.STATE_EXPANDABLE) \
                or state.contains(pyatspi.STATE_COLLAPSED):
                 if state.contains(pyatspi.STATE_VISIBLE):
                     count += 1
-                obj = obj.parent
+                newObj = newObj.parent
             else:
                 break
 
         return count - 1
 
+    def isSameObject(self, obj1, obj2):
+        """Compares two objects to determine if they are functionally
+        the same object. This is needed because some applications and
+        toolkits kill and replace accessibles."""
+
+        if (obj1 == obj2):
+            return True
+        elif (not obj1) or (not obj2):
+            return False
+        elif (obj1.name != obj2.name) or (obj1.childCount != obj2.childCount):
+            return False
+
+        # This is to handle labels in trees. In some cases the default
+        # script's method gives us false positives; other times false
+        # negatives.
+        #
+        if obj1.getRole() == obj2.getRole() == pyatspi.ROLE_LABEL:
+            try:
+                ext1 = obj1.queryComponent().getExtents(0)
+                ext2 = obj2.queryComponent().getExtents(0)
+            except:
+                pass
+            else:
+                if ext1.x == ext2.x and ext1.y == ext2.y \
+                   and ext1.width == ext2.width and ext1.height == ext2.height:
+                    return True
+
+        return default.Script.isSameObject(self, obj1, obj2)
+
     def onFocus(self, event):
         """Called whenever an object gets focus.
 
@@ -132,6 +199,19 @@ class Script(default.Script):
 
         role = event.source.getRole()
 
+        # Ignore bogus/senseless focus claims because they just cause us to
+        # be more chatty as a result of the event and then later as a result
+        # of the changed locusOfFocus.
+        #
+        if role == pyatspi.ROLE_ROOT_PANE:
+            return
+
+        if role == pyatspi.ROLE_PAGE_TAB_LIST and orca_state.locusOfFocus \
+           and orca_state.locusOfFocus.getRole() == pyatspi.ROLE_PAGE_TAB \
+           and orca_state.locusOfFocus.getState().\
+               contains(pyatspi.STATE_FOCUSED):
+            return
+
         if role == pyatspi.ROLE_MENU:
             # Override default.py's onFocus decision to ignore focus
             # events on MENU items with selected children.  This is
@@ -157,6 +237,8 @@ class Script(default.Script):
         - event: the Event
         """
 
+        self.lastDescendantChangedSource = event.source
+
         # In Java comboboxes, when the list of options is popped up via
         # an up or down action, control (but not focus) goes to a LIST
         # object that manages the descendants.  So, we detect that here
@@ -210,6 +292,14 @@ class Script(default.Script):
         - event: the Event
         """
 
+        # Avoid doing this with objects that manage their descendants
+        # because they'll issue a descendant changed event. (Note: This
+        # equality check is intentional; isSameObject() is especially
+        # thorough with trees and tables, which is not performant.
+        #
+        if event.source == self.lastDescendantChangedSource:
+            return
+
         # We treat selected children as the locus of focus. When the
         # selection changes in a list we want to update the locus of
         # focus. If there is no selection, we default the locus of
@@ -289,7 +379,10 @@ class Script(default.Script):
         # We'll ignore value changed events for Java's toggle buttons since
         # they also send a redundant object:state-changed:checked event.
         #
-        if event.source.getRole() == pyatspi.ROLE_TOGGLE_BUTTON:
+        ignoreRoles = [pyatspi.ROLE_TOGGLE_BUTTON,
+                       pyatspi.ROLE_RADIO_BUTTON,
+                       pyatspi.ROLE_CHECK_BOX]
+        if event.source.getRole() in ignoreRoles:
             return
 
         # Java's SpinButtons are the most caret movement happy thing
