@@ -186,7 +186,11 @@ userCustomizableSettings = [
     "wrappedStructuralNavigation",
     "presentRequiredState",
     "brailleRequiredStateString",
-    "speechRequiredStateString"
+    "speechRequiredStateString",
+    "chatMessageVerbosity",
+    "chatPrefixMessageWithName",
+    "chatAnnounceBuddyTyping",
+    "chatSeparateRoomHistories",
 ]
 
 # The name of the module that hold the user interface for the main window
@@ -228,6 +232,11 @@ DESKTOP_MODIFIER_KEYS = ["Insert", "KP_Insert"]
 LAPTOP_MODIFIER_KEYS  = ["Caps_Lock"]
 orcaModifierKeys      = DESKTOP_MODIFIER_KEYS
 
+# If True, some validation of the keybindings will be done to check for
+# missing keysyms and duplicate keys used for different operations.
+#
+validateKeyBindings = False
+
 # A new modifier to use (set by the press of any key in the
 # orcaModifierKeys list) to represent the Orca modifier.
 #
@@ -709,8 +718,8 @@ allTextAttributes = \
     "pixels-below-lines:; pixels-inside-wrap:; right-margin:; rise:; " \
     "scale:; size:; stretch:; strikethrough:false; style:normal; " \
     "text-decoration:none; text-rotation:0; text-shadow:none; " \
-    "underline:none; variant:; vertical-align:baseline; weight:400; " \
-    "wrap-mode:; writing-mode:lr-tb;"
+    "text-spelling:none; underline:none; variant:; " \
+    "vertical-align:baseline; weight:400; wrap-mode:; writing-mode:lr-tb;"
 
 # The default set of text attributes to speak to the user. Specific
 # application scripts (or individual users can override these values if
@@ -723,7 +732,7 @@ allTextAttributes = \
 enabledSpokenTextAttributes = \
     "size:; family-name:; weight:400; indent:0; underline:none; " \
     "strikethrough:false; justification:left; style:normal; " \
-    "paragraph-style:;"
+    "paragraph-style:; text-spelling:none;"
 
 # The default set of text attributes to be brailled for the user. Specific
 # application scripts (or individual users can override these values if
@@ -735,7 +744,8 @@ enabledSpokenTextAttributes = \
 
 enabledBrailledTextAttributes = \
     "size:; family-name:; weight:400; indent:0; underline:none; " \
-    "strikethrough:false; justification:left; style:normal;"
+    "strikethrough:false; justification:left; style:normal; " \
+    "text-spelling:none;"
 
 # Text Attributes Braille Indicator (see textAttributesBrailleIndicator).
 # The values represent the character to be used in the attrOr
@@ -859,12 +869,10 @@ useBonoboMain           = True
 asyncMode               = True
 
 # A list of toolkits whose events we need to process synchronously.
-# The only one right now is the Java toolkit (see bug #531869), but
-# we put this here to allow more toolkits to be more easily added
-# and for Java to be removed if issues with asynchronous processing
-# of its events are ever resolved.
+# This was originally added for the Java toolkit (see bug #531869), but
+# we put this here to allow more toolkits to be more easily added.
 #
-synchronousToolkits     = ["J2SE-access-bridge"]
+synchronousToolkits     = []
 
 # If True, we output debug information for the event queue.  We
 # use this in addition to log level to prevent debug logic from
@@ -894,6 +902,26 @@ timeoutCallback         = None # Set by orca.py:init to orca.timeout
 #
 doubleClickTimeout = 0.5
 
+# Available options indicating which chat messages Orca should speak.
+#
+CHAT_SPEAK_ALL             = 0
+CHAT_SPEAK_ALL_IF_FOCUSED  = 1
+CHAT_SPEAK_FOCUSED_CHANNEL = 2
+
+chatMessageVerbosity = CHAT_SPEAK_ALL
+
+# Whether we prefix chat room messages with the name of the chat room.
+#
+chatSpeakRoomName = False
+
+# Whether we announce when a buddy is typing.
+#
+chatAnnounceBuddyTyping = False
+
+# Whether we provide chat room specific message histories.
+#
+chatRoomHistories = False
+
 # Obtain/set information regarding whether accessibility is enabled
 # or not.
 #
@@ -1130,6 +1158,9 @@ setScriptMapping(re.compile('Pidgin'), "pidgin")
 #setScriptMapping(re.compile(_('gnome-help')), "yelp")
 setScriptMapping(re.compile('gnome-help'), "yelp")
 
+# Gnome-calculator is symbolically linked to gcalctool so we associate it here.
+setScriptMapping(re.compile('gnome-calculator'), "gcalctool")
+
 # This is for development builds of Banshee
 setScriptMapping(re.compile('Nereid'), "Banshee")
 
@@ -1138,6 +1169,10 @@ setScriptMapping(re.compile('Nereid'), "Banshee")
 #
 setScriptMapping(re.compile('ddu.py'), "ddu")
 
+# Strip off the extra 'py' that Gajim includes as part of its name.
+#
+setScriptMapping(re.compile('gajim.py'), "gajim")
+
 # Show deprecated messeges in debug output.
 # Set this to True to help find potential pyatspi porting problems
 #
